@@ -9,16 +9,14 @@ use Carp qw(croak);
 use Sub::Accessor::Small ();
 
 our $AUTHORITY = 'cpan:TOBYINK';
-our $VERSION   = '0.006';
+our $VERSION   = '0.007';
 our @EXPORT    = qw/ lexical_has /;
 our @ISA       = qw/ Sub::Accessor::Small /;
 
 sub _generate_lexical_has : method
 {
 	my $me = shift;
-	my (undef, undef, $export_opts) = @_;
-	
-	my $code = sub { $me->lexical_has($export_opts, @_) };
+	my $code = $me->_generate_has(@_);
 	$code = Sub::Name::subname("$me\::lexical_has", $code)
 		if Sub::Accessor::Small::HAS_SUB_NAME;
 	return $code;
@@ -30,51 +28,94 @@ sub lexical_has : method
 	goto $has;
 }
 	
-sub _inline_to_coderef : method
+sub inline_to_coderef : method
 {
 	my $me = shift;
-	my ($method_type, $code, $name, $uniq, $opts) = @_;
-	my $coderef = $me->SUPER::_inline_to_coderef(@_);
-	Sub::Accessor::Small::HAS_SUB_NAME && $opts->{package} && defined($name)
-		? Sub::Name::subname("$opts->{package}\::__LEXICAL__[$name]", $coderef)
+	my ($method_type, $code) = @_;
+	my $coderef = $me->SUPER::inline_to_coderef(@_);
+	Sub::Accessor::Small::HAS_SUB_NAME && $me->{package} && defined($me->{slot})
+		? Sub::Name::subname("$me->{package}\::__LEXICAL__[$me->{slot}]", $coderef)
 		: $coderef
 }
 
-sub _accessor_kind : method
+sub accessor_kind : method
 {
 	return 'lexical';
 }
 
-sub _canonicalize_opts : method
+sub canonicalize_is : method
 {
 	my $me = shift;
-	my ($name, $uniq, $opts) = @_;
-	$me->SUPER::_canonicalize_opts(@_);
+	
+	if ($me->{is} eq 'rw')
+	{
+		$me->{accessor} = \(my $tmp)
+			if !exists($me->{accessor});
+	}
+	elsif ($me->{is} eq 'ro')
+	{
+		$me->{reader} = \(my $tmp)
+			if !exists($me->{reader});
+	}
+	elsif ($me->{is} eq 'rwp')
+	{
+		$me->{reader} = \(my $tmp1)
+			if !exists($me->{reader});
+		$me->{writer} = \(my $tmp2)
+			if !exists($me->{writer});
+	}
+	elsif ($me->{is} eq 'lazy')
+	{
+		$me->{reader} = \(my $tmp)
+			if !exists($me->{reader});
+		$me->{lazy} = 1
+			if !exists($me->{lazy});
+		$me->{builder} = 1
+			unless $me->{builder} || $me->{default};
+	}
+}
 
-	if (defined $opts->{init_arg})
+sub canonicalize_opts : method
+{
+	my $me = shift;
+	$me->SUPER::canonicalize_opts(@_);
+
+	if (defined $me->{init_arg})
 	{
-		croak("Invalid init_arg; private attributes cannot be initialized in the constructor");
+		croak("Invalid init_arg=>defined; private attributes cannot be initialized in the constructor");
 	}
 	
-	if ($opts->{required})
+	if ($me->{required})
 	{
-		croak("Invalid required; private attributes cannot be initialized in the constructor");
+		croak("Invalid required=>1; private attributes cannot be initialized in the constructor");
 	}
 	
-	if (defined $opts->{lazy} and not $opts->{lazy})
+	if (defined $me->{lazy} and not $me->{lazy})
 	{
-		croak("Invalid lazy; private attributes cannot be eager");
+		croak("Invalid lazy=>0; private attributes cannot be eager");
 	}
 	
 	for my $type (qw/ reader writer accessor clearer predicate /)
 	{
-		if (defined($opts->{$type}) and not ref($opts->{$type}) eq q(SCALAR))
+		if (defined($me->{$type}) and not ref($me->{$type}) eq q(SCALAR))
 		{
-			croak("Expected $type to be a scalar ref; not '$opts->{$type}'");
+			croak("Expected $type to be a scalar ref; not '$me->{$type}'");
 		}
 	}
 }
 
+sub expand_handles
+{
+	my $me = shift;
+	
+	if (ref($me->{handles}) eq q(ARRAY))
+	{
+		return @{$me->{handles}};
+	}
+	
+	croak "Expected delegations to be a reference to an array; got $me->{handles}";
+}
+
 1;
 
 __END__
